<?php

namespace FiloBlu\ExtEbizmartsMailChimp\Helper;

class Data extends \Ebizmarts\MailChimp\Helper\Data {

    const XML_PATH_CRON = 'mailchimp/general/enable_cron';
    const XML_PATH_CRON_SYNC_STATUS = 'mailchimp/general/enable_cron_sync_status';

    private $_customer;
    private $_storeManager;
    private $_subscriberInfo;

    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Store\Model\StoreManagerInterface $storeManager, \Ebizmarts\MailChimp\Model\Logger\Logger $logger, \Magento\Customer\Model\GroupRegistry $groupRegistry, \Magento\Framework\App\State $state, \Magento\Framework\Module\ModuleList\Loader $loader, \Magento\Config\Model\ResourceModel\Config $config, \Mailchimp $api, \Magento\Customer\Model\ResourceModel\CustomerRepository $customer, \Ebizmarts\MailChimp\Model\MailChimpErrors $mailChimpErrors, \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerceFactory $mailChimpSyncEcommerce, \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerce $mailChimpSyncE, \Ebizmarts\MailChimp\Model\MailChimpSyncBatches $syncBatches, \Ebizmarts\MailChimp\Model\MailChimpStoresFactory $mailChimpStoresFactory, \Ebizmarts\MailChimp\Model\MailChimpStores $mailChimpStores, \Magento\Framework\Encryption\Encryptor $encryptor, \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollection, \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollection, \Magento\Customer\Api\AddressRepositoryInterface $addressRepositoryInterface, \FiloBlu\Newsletter\Model\SubscriberInfo $subscriberInfo, \Magento\Newsletter\Model\Subscriber $subscriber, \FiloBlu\ExtEbizmartsMailChimp\Model\Logger\LoggerImport $loggerImport
    ) {
        $this->_customer = $customer;
        $this->_storeManager = $storeManager;
        $this->_subscriberInfo = $subscriberInfo;
        $this->_subscriber = $subscriber;
        $this->_loggerImport = $loggerImport;
        parent::__construct($context, $storeManager, $logger, $groupRegistry, $state, $loader, $config, $api, $customer, $mailChimpErrors, $mailChimpSyncEcommerce, $mailChimpSyncE, $syncBatches, $mailChimpStoresFactory, $mailChimpStores, $encryptor, $subscriberCollection, $customerCollection, $addressRepositoryInterface);    
        
    }

    public function isCronEnabled() {
        return ($this->isMailChimpEnabled() && $this->getConfigValue(self::XML_PATH_CRON));
    }
    
    public function isCronSyncStatusEnabled() {
        return ($this->isMailChimpEnabled() && $this->getConfigValue(self::XML_PATH_CRON_SYNC_STATUS));
    }

    public function getMergeVars($subscriber, $isCustomer = false, $loadFromTable = false) {

        $email = $subscriber->getEmail();

        if ($isCustomer) {
            $subscriberType = "CUSTOMER";
        } else {
            $subscriberType = "GUEST";

            try {
                $this->_customer->get($email, $this->_storeManager->getStore($subscriber->getStoreId())->getWebsite()->getId());
                $subscriberType = "CUSTOMER";
            } catch (\Exception $ex) {

                // No customer found
            }
        }

        $merge_vars = array();

        $mergeVars = unserialize($this->getConfigValue(self::XML_PATH_MAPPING));

        if ($loadFromTable) {
            $dataMerge = $this->_subscriberInfo->load($subscriber->getId())->getData();
        } else {
            $dataMerge = $this->_request->getPost();
        }

        foreach ($mergeVars as $map) {
            $merge_vars = array_merge($merge_vars, $this->_getMergeVarsValue($map, $dataMerge, $loadFromTable));
        }

        $merge_vars["TYPE"] = $subscriberType;

        if ($subscriber->getMailchimpId()) {
            $merge_vars["CHIMPID"] = $subscriber->getMailchimpId();
        }

        return $merge_vars;
    }

    protected function _getMergeVarsValue($map, $dataMerge, $loadFromTable = false) {
        $merge_vars = array();

        if ($loadFromTable) {
            $customAtt = $map['dbcolumnname'];
        } else {
            $customAtt = $map['magento'];
        }

        $chimpTag = $map['mailchimp'];

        if ($chimpTag && $customAtt) {
            $key = strtoupper($chimpTag);

            if (isset($dataMerge[$customAtt])) {
                if (is_array($dataMerge[$customAtt])) {
                    $valStr = "";
                    foreach ($dataMerge[$customAtt] as $val) {
                        if (is_string($val)) {
                            if ($valStr) {
                                $valStr .= ' | ' . $val;
                            } else {
                                $valStr = $val;
                            }
                        }
                    }

                    if ($valStr) {
                        $merge_vars[$key] = $valStr;
                    }
                } else {
                    if ($dataMerge[$customAtt]) {
                        $merge_vars[$key] = $dataMerge[$customAtt];
                    }
                }
            }

            return $merge_vars;
        }
    }

    public function sendToMailchimp($subscribersIds) {
        
        $status = '';
        $count = 0;

        $countOk = 0;
        $countStatusNotAllowed = 0;
        $countError = 0;
        $this->_loggerImport->info(" #### START REQUEST COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $this->_loggerImport->info("processing subscriber id " . $subscriberId);

            $subscriber = $this->_subscriber->load($subscriberId);

            if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {

                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVars($subscriber, false, true);

//                if ($mergeVars) {
//
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }

                try {
                    $md5HashEmail = md5(strtolower($email));
                    $api = $this->getApi($subscriber->getStoreId());
                    if ($subscriber->getMailchimpId()) {

                        $this->_loggerImport->info("update subscriber id " . $subscriberId);
                        $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    } else {
                        $this->_loggerImport->info("new subscriber id " . $subscriberId);
                        $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                        if (isset($return["id"])) {
                            $subscriber->setMailchimpId($return["id"]);
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return["id"]);
                        } else {
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                        }
                    }

                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            } else {
                $this->_loggerImport->info("status not allowed for subscriber id" . $subscriberId);
                $countStatusNotAllowed++;
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " / Result NotAllowed: " . $countStatusNotAllowed . " #### ");

        return $count;
    }

    public function sendToMailchimpOnlyNew($subscribersIds) {
        
        $status = '';
        $count = 0;

        $countOk = 0;
        $countError = 0;

        $this->_loggerImport->info(" #### START REQUEST ONLY NEW COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $subscriber = $this->_subscriber->load($subscriberId);

            if (!$subscriber->getMailchimpId() && ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED)) {

                $this->_loggerImport->info("processing subscriber id " . $subscriberId);

                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVars($subscriber, false, true);

//                if ($mergeVars) {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }

                try {

                    $api = $this->getApi($subscriber->getStoreId());
                    
                    $md5HashEmail = md5(strtolower($email));

                    $this->_loggerImport->info("new subscriber id " . $subscriberId);
                    $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                    if (isset($return["id"])) {
                        $subscriber->setMailchimpId($return["id"]);
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return["id"]);
                    } else {
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                    }


                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " #### ");

        return $count;
    }
    
    public function updateMailChimpIdFromMageMonkeyId($subscribersIds) {
        
        $count = 0;
        $countOk = 0;
        $countError = 0;

        $this->_loggerImport->info(" #### START REQUEST UPDATE MAILCHIMP ID FROM MAGEMONKEY ID COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $subscriber = $this->_subscriber->load($subscriberId);

            if (!$subscriber->getMailchimpId() && $subscriber->getMagemonkeyId()) {

                $this->_loggerImport->info("processing subscriber id " . $subscriberId);

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                try {

                    $subscriber->setMailchimpId($subscriber->getMagemonkeyId());
                    $subscriber->setMagemonkeyId('');

                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
                }
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " #### ");

        return $count;
    }
    
    public function syncStatusFromMailChimpToMagento($subscribersIds) {
        
        $count = 0;

        $countOk = 0;
        $countError = 0;
        $countSkipped = 0;
        $this->_loggerImport->info(" #### START REQUEST SYNC MAILCHIMP TO MAGENTO COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $this->_loggerImport->info("processing subscriber id " . $subscriberId);

            $subscriber = $this->_subscriber->load($subscriberId);

            if ($subscriber->getMailchimpId()) {

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                try {

                    $api = $this->getApi($subscriber->getStoreId());
                    
                    $md5HashEmail = md5(strtolower($email));
                  
                    $return = $api->lists->members->get($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail);

                    if ($return["status"] == "subscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                        $countSkipped ++;
                        continue;
                    }

                    if ($return["status"] == "unsubscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {
                        $countSkipped ++;
                        continue;
                    }

                    // Subscribe to magento
                    if ($return["status"] == "subscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {
                        $subscriber->setStatus(\Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED);
                        $this->_loggerImport->info($email . " changed status from unsubscribed to subscribed");
                        $subscriber->save();
                        $countOk++;
                        $count++;
                        continue;
                    }

                    // Unsubscribed to magento
                    if ($return["status"] == "unsubscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                        $subscriber->setStatus(\Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED);
                        $this->_loggerImport->info($email . " changed status from subscribed to unsubscribed");
                        $subscriber->save();
                        $countOk++;
                        $count++;
                        continue;
                    }

                    $countSkipped ++;
//                    $count++;
                } catch (\Exception $e) {

                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            } else {
                $countSkipped ++;
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " / Result Skipped: " . $countSkipped . " #### ");

        return $count;
    }

}
